import pandas as pd
import matplotlib.pyplot as plt
import numpy as np

matriz = pd.read_excel("matriz-indicadores.xlsx")

formulario = pd.read_excel("formulario.xlsx")
catalogo = pd.read_excel("catalogo.xlsx")
formulario.columns = catalogo.label

insitu = pd.read_excel('formulario.xlsx', sheetname='insitu')
insitu_cat = pd.read_excel('catalogo.xlsx', sheetname='insitu')
insitu.columns = insitu_cat['label']
formulario = formulario.merge(insitu, on='institucion')

online = pd.read_excel('formulario.xlsx', sheetname='online')
online_cat = pd.read_excel('catalogo.xlsx', sheetname='online')
online.columns = online_cat['label']
formulario = formulario.merge(online, on='institucion')


cols = ['viacomm', 'reqdenun', 'confidencialidad', 'tipodncia', 'viasuger',
        'iniinvest', 'docnorm', 'afirm01', 'lugrecep', 'viaproblem', 
        'rectecn', 'beneficios', 'proteccion', 'oportunidad', 'afirm02',
        'lstacciones', 'tpoadminefect', 'ofiplan', 'intacciones',
        'acdifofic', 'viasegusr', 'usractual', 'horario', 'tpoatencion',
        'infciud', 'concciud', 'infclasif', 'medorient', 'linprotec']

for index in cols:
    formulario[index] = formulario[index].str.split(', ')

# Creando un nuevo dataframe
data = pd.DataFrame(formulario.institucion)

meta = {}

def annotate(ax):
    for p in ax.patches: 
        ax.annotate(np.round(p.get_height(),decimals=2), 
                    (p.get_x()+p.get_width()/2., p.get_height()), 
                    ha='center', va='center', xytext=(0, 10), 
                    textcoords='offset points')

def generateSet(x):
    respuestas = set()
    for el in x:
        for item in el:
            respuestas.add(item)
    return respuestas

def is_match(x, text):
    for el in x:
        if el.find(text) != -1:
            return 1
    return 0

def is_otra(x, variables):
    tmp = list(set(x))
    for index in variables[0:len(variables) - 1]:
        for el in tmp:
            if el.find(index) != -1:
                tmp.remove(el)
    if len(tmp) > 0:
        return 1
    else:
        return 0
    
def meta_check_len(index):
    return len(meta[index]['vars']) == len(meta[index]['labels'])
    
def generateColumns(index):
    for i in range(len(meta[index]['vars']) - 1):
        data[meta[index]['vars'][i]] = \
            formulario[index].apply(lambda x: \
                is_match(x, meta[index]['labels'][i]))
    data[meta[index]['vars'][-1]] = formulario[index].apply(lambda x: \
        is_otra(x, meta[index]['labels']))
    return data[['institucion'] + meta[index]['vars']]
    

def meanColumns(index):
    n = len(meta[index]['vars']) - 1
    data[index] = np.zeros(len(data))
    for i in range(n):
        data[index] += data[meta[index]['vars'][i]]
    data[index] /= n
    data[index] = data[index].round(4)
    
def discountColumns(index):
    n = len(meta[index]['vars']) - 1
    data[index] = np.repeat(n, len(data))
    for i in range(n):
        data[index] -= data[meta[index]['vars'][i]]
    data[index] /= n
    data[index] = data[index].round(4)
    
def rankByOrderColumns(index):
    res = np.zeros(len(data))
    n = len(meta[index]['vars']) - 2
    step = 1 / n
    for i in range(n):
        for j in range(len(data)):
            if data[[meta[index]['vars'][n - i - 1]]].as_matrix()[j] == 1:
                res[j] = step * (i + 1)
    data[index] = res.round(4)
    return data[index]
    
def oneIfAnyColumns(index):
    n = len(meta[index]['vars']) - 1
    data[index] = np.zeros(len(data))
    for i in range(n):
        data[index] += data[meta[index]['vars'][i]]
    data[index] = data[index].apply(lambda x: 1 if x >= 1 else 0)
    
def showColumns(index, title):
    res = data[meta[index]['vars']].sum()
    res = pd.DataFrame(res / len(data) * 100)
    ax = res.plot(kind='bar', title=title, legend=False, alpha=0.5, color='#01b1af')
    ax.set_xticklabels(meta[index]['labels'])
    ax.set_ylim(0, 1)
    ax.set_ylabel('Porcentaje')
    annotate(ax)
    ax.get_figure().savefig(index + '_by_column.png')
    return res
    
def aggregateColumns(index, title):
    res = data[index].groupby(data[index]).count()
    ax = res.plot(kind='bar', title=title, alpha=0.5, color='#01b1af')
    ax.set_xlabel('Resultados')
    ax.set_ylabel('Frecuencia')
    annotate(ax)
    ax.get_figure().savefig(index + '_aggregate.png')
    return res
    
def generateStats(index):
    meta[index]['mean'] = data[index].mean()
    meta[index]['median'] = data[index].median()
    print('Mediana:  %.4f' % data[index].median())
    print('Promedio: %.4f' % data[index].mean())
    print('Desviación estandar: %.4f' % data[index].std())
    
def byOffice(index, title):
    res = data[['institucion', index]]
    ax = res.plot(kind='bar', title=title, legend=False, alpha=0.5, color='#01b1af')
    ax.set_xticklabels(data['institucion'])
    ax.set_ylim(0, 1)
    annotate(ax)
    ax.get_figure().savefig(index + '_by_office.png')
    return res
    
def esSi(x):
    if x[0:2].lower().find('si') != -1:
        return 1
    elif x[0:2].lower().find('sí') != -1:
        return 1
    else:
        return 0

def yesNoColumn(index):
    data[index] = formulario[index].apply(esSi)
    return data[index]
    


